@@ -37,7 +37,7 @@ struct pctrl_info {
     double factor;
     int sktype;
     long skval;
-    unsigned char paused, brk;
+    unsigned char paused, brk, echo;
 };
 
 extern int pctrl_init(void);
