@@ -28,9 +28,9 @@ include/rdsh.h - Shared definitions for RPLD/INFO/RPLINFO
 #include <pthread.h>
 #include <stdint.h>
 #include <libHX.h>
-#if !defined(_LIBHX_H) || _LIBHX_H < 20041024
+#if !defined(_LIBHX_H) || _LIBHX_H < 20041109
 #    error --------------------------------------------
-#    error You need a newer version of libHX (20041024)
+#    error You need a newer version of libHX (20041109)
 #    error --------------------------------------------
 #endif
 
@@ -40,7 +40,7 @@ include/rdsh.h - Shared definitions for RPLD/INFO/RPLINFO
 struct GOptmap_t {
     int syslog, verbose;
     long user_id;
-    const char *infod_port;
+    const char *infod_port, *ofmt;
 };
 
 struct Statmap_t {
@@ -76,8 +76,6 @@ enum {
 
     // req(const char *)
     IFT_STR_START = 0x80,
-    IFP_ENABUSER,
-    IFP_DEACUSER,
     IFT_STR_STOP,
 
     NF_NEWLINE = 1 << 0,
@@ -100,7 +98,7 @@ extern char *G_devname_nm(uint32_t, char *, size_t);
 extern char *G_devname_fs(uint32_t, char *, size_t);
 extern struct tty *get_tty(uint32_t);
 extern void log_close(struct tty *);
-extern int notify(int, unsigned int, const char *, ...);
+extern void notify(int, const char *, ...);
 extern ssize_t send_wait(int, const void *, size_t, int);
 
 #endif // TTYRPLD_RDSH_H
