@@ -33,7 +33,6 @@ include/rpl_packet.h - Packet definitions for RPLD and RPLDEV
 #endif
 
 enum {
-    MAGIC_2_4 = 0xFF,
     MAGIC_2_6 = 0xEE,
     MAGIC_SIG = MAGIC_2_6,
 };
@@ -48,26 +47,26 @@ enum {
     EVT_INIT   = 'i',
     EVT_DEINIT = 'd',
     EVT_IDENT  = 240,
-    EVT_IGNORE = 255,
+    EVT_max,
 };
 
 struct rpld_packet {
-    uint32_t dev, dev2 PACKED;
-    uint16_t size PACKED;
-    uint8_t event PACKED, magic PACKED;
+    uint32_t dev, dev2;
+    uint16_t size;
+    uint8_t event, magic;
 } PACKED;
 
 struct mem_packet {
-    uint32_t dev, dev2 PACKED;
-    uint16_t size PACKED;
-    uint8_t event PACKED, magic PACKED;
-    struct timeval tv PACKED;
+    uint32_t dev, dev2;
+    uint16_t size;
+    uint8_t event, magic;
+    struct timeval tv;
 } PACKED;
 
 struct disk_packet {
     uint16_t size;
-    uint8_t event PACKED, magic PACKED;
-    struct timeval tv PACKED;
+    uint8_t event, magic;
+    struct timeval tv;
 } PACKED;
 
 #endif /* RPL_PACKET_H */
