@@ -42,12 +42,12 @@ user/infod.c - INFO socket for RPLD
 
 // Function prototypes
 static void *client_thread(void *);
-static void flush_table(void);
 static void set_session_status(uint32_t, int);
 static void getinfo_text(uint32_t, int);
 static void getinfo(uint32_t, int);
 static void zero_counters(void);
 
+inline static const char *basename_pp(const char *, const char *);
 static void block_signals(void);
 static void print_entry_text(int, struct tty *);
 static void print_entry(int, struct tty *);
@@ -61,7 +61,7 @@ static int Svfd = -1;
 void infod_init(void) {
     /* bind() in unix_server() needs to be called as root (since in the default
     case we will create the socket in /var/run/), so init_infod() must come
-    before setuid(). So it's an extra function. */
+    before setuid(). */
     Svfd = unix_server(GOpt.infod_port);
 
     /* Limit the connections to the user owning the socket (and root). The
@@ -114,9 +114,6 @@ static void *client_thread(void *arg) {
 
         switch(req) {
             // Voids
-            case IFP_FLUSH:
-                flush_table();
-                break;
             case IFP_ZERO:
                 zero_counters();
                 break;
@@ -127,12 +124,9 @@ static void *client_thread(void *arg) {
                 break;
             case IFP_REMOVE: {
                 struct tty *tty;
-
                 pthread_mutex_lock(&Ttys_lock);
-                tty = get_tty(gint);
+                if((tty = get_tty(gint)) != NULL) { log_close(tty); }
                 pthread_mutex_unlock(&Ttys_lock);
-
-                if(tty != NULL) { log_close(tty); }
                 break;
             }
             case IFP_GETINFO:
@@ -147,10 +141,9 @@ static void *client_thread(void *arg) {
 
             // Other
             default:
-                notify(LOG_WARNING, NF_NEWLINE, "Unknown request %d, closing"
+                notify(LOG_WARNING, "infod: Unknown request %d, closing"
                  " connection to not run into conversion problems.\n", req);
                 goto clthr__exit;
-                break;
         }
 
         send_wait(fd, &zero, sizeof(uint32_t), 0);
@@ -162,44 +155,23 @@ clthr__exit:
 }
 
 //-----------------------------------------------------------------------------
-static void flush_table(void) {
-    struct HXbtree_node *nd;
-    struct HXbtrav *travp;
-
-    pthread_mutex_lock(&Ttys_lock);
-    travp = HXbtrav_init(Ttys, NULL);
-    while((nd = HXbtraverse(travp)) != NULL) {
-        struct tty *tty = nd->data;
-        if(tty->status != IFP_DEACTIVATE && tty->fd == -1) {
-            HXbtree_del(Ttys, (void *)tty->dev);
-            if(tty->file != NULL) { free(tty->file); }
-            free(tty);
-        }
-    }
-
-    pthread_mutex_unlock(&Ttys_lock);
-    HXbtrav_free(travp);
-    return;
-}
-
 static void getinfo_text(uint32_t dev, int fd) {
+    pthread_mutex_lock(&Ttys_lock);
     if(dev == 0) {
         // No device given, send info about RPLD and all ttys
         struct HXbtree_node *nd;
         struct HXbtrav *travp;
 
         skprintf(fd,
-          "%2s %8s %-7s %9s %9s %4s %s\n"
+          "AC   DEVICE TTY      BYTES IN       OUT FILENAME\n"
           "==========================================================================\n"
           "  [---,---] *       %9llu %9llu    -\n"
           "            IOCD: %zu/%zu/%zu/%zu  RW: %zu/%zu  I: %zu  B: %zu\n"
           "--------------------------------------------------------------------------\n",
-          "AC", "DEVICE", "TTY", "BYTES IN", "OUT", "FD", "FILENAME",
           Stats.in, Stats.out, Stats.init, Stats.open, Stats.close,
           Stats.deinit, Stats.read, Stats.write, Stats.ioctl, Stats.badpack
         );
 
-        pthread_mutex_lock(&Ttys_lock);
         travp = HXbtrav_init(Ttys, NULL);
 
         while((nd = HXbtraverse(travp)) != NULL) {
@@ -207,55 +179,51 @@ static void getinfo_text(uint32_t dev, int fd) {
         }
 
         skprintf(fd, "--------------------------------------------------------------------------\n");
-        pthread_mutex_unlock(&Ttys_lock);
         HXbtrav_free(travp);
-        return;
     } else {
         struct tty *tty;
-
-        pthread_mutex_lock(&Ttys_lock);
-        tty = get_tty(dev);
-        pthread_mutex_unlock(&Ttys_lock);
-
-        if(tty == NULL) { return; }
+        if((tty = get_tty(dev)) == NULL) {
+            pthread_mutex_unlock(&Ttys_lock);
+            return;
+        }
         print_entry_text(fd, tty);
     }
 
+    pthread_mutex_unlock(&Ttys_lock);
     return;
 }
 
 static void getinfo(uint32_t dev, int fd) {
+    pthread_mutex_lock(&Ttys_lock);
     if(dev == 0) {
         // No device given, send info about RPLD and all ttys
         struct HXbtree_node *nd;
         struct HXbtrav *travp;
 
-        skprintf(fd, "%zu %zu %zu %zu %zu %zu %llu %llu %zu %zu\n",
+        skprintf(fd,
+          "format 2\n"
+          "%zu %zu %zu %zu %zu %zu %llu %llu %zu %zu\n",
           Stats.init, Stats.open, Stats.close, Stats.deinit, Stats.read,
           Stats.write, Stats.in, Stats.out, Stats.ioctl, Stats.badpack
         );
 
-        pthread_mutex_lock(&Ttys_lock);
         travp = HXbtrav_init(Ttys, NULL);
 
         while((nd = HXbtraverse(travp)) != NULL) {
             print_entry(fd, nd->data);
         }
 
-        pthread_mutex_unlock(&Ttys_lock);
         HXbtrav_free(travp);
-        return;
     } else {
         struct tty *tty;
-
-        pthread_mutex_lock(&Ttys_lock);
-        tty = get_tty(dev);
-        pthread_mutex_unlock(&Ttys_lock);
-
-        if(tty == NULL) { return; }
+        if((tty = get_tty(dev)) == NULL) {
+            pthread_mutex_unlock(&Ttys_lock);
+            return;
+        }
         print_entry(fd, tty);
     }
 
+    pthread_mutex_unlock(&Ttys_lock);
     return;
 }
 
@@ -289,12 +257,24 @@ static void zero_counters(void) {
     Stats.write = Stats.ioctl = Stats.badpack = 0;
     Stats.in = Stats.out = 0;
 
-    pthread_mutex_unlock(&Ttys_lock);
     HXbtrav_free(travp);
+    pthread_mutex_unlock(&Ttys_lock);
     return;
 }
 
 //-----------------------------------------------------------------------------
+inline static const char *basename_pp(const char *s, const char *t) {
+    const char *p;
+
+    if((p = strchr(t, '%')) != NULL) {
+        if((p = HX_strbchr(t, p, '/')) != NULL) { return s + (p - t) + 1; }
+    } else if((p = strrchr(s, '/')) != NULL) {
+        return p + 1;
+    }
+
+    return s;
+}
+
 static void block_signals(void) {
     sigset_t set;
     sigfillset(&set);
@@ -313,8 +293,10 @@ static void print_entry_text(int fd, struct tty *tty) {
     }
 
     G_devname_nm(tty->dev, buf, sizeof(buf));
-    skprintf(fd, "%c [%3ld,%3ld] %-7s %9zu %9zu %4d %s\n", status,
-     KD26_PARTS(tty->dev), buf, tty->in, tty->out, tty->fd, tty->file);
+    skprintf(fd, "%c [%3ld,%3ld] %-7s %9zu %9zu %s\n", status,
+      KD26_PARTS(tty->dev), buf, tty->in, tty->out,
+      basename_pp(tty->file, GOpt.ofmt)
+    );
     return;
 }
 
@@ -323,14 +305,14 @@ static void print_entry(int fd, struct tty *tty) {
 
     switch(tty->status) {
         case IFP_ACTIVATE:   status = 'A'; break;
-        case IFP_DEACTIVATE: status = 'J'; break;
-        case IFP_DEACTIVSES: status = 'D'; break;
+        case IFP_DEACTIVATE: status = 'D'; break;
+        case IFP_DEACTIVSES: status = 'S'; break;
         default:             status = '?'; break;
     }
 
     G_devname_nm(tty->dev, buf, sizeof(buf));
-    skprintf(fd, "%c %ld %ld %s %zu %zu %d %s\n", status, KD26_PARTS(tty->dev),
-     buf, tty->in, tty->out, tty->fd, tty->file);
+    skprintf(fd, "%c %ld %ld %s %zu %zu %s\n",
+     status, KD26_PARTS(tty->dev), buf, tty->in, tty->out, tty->file);
     return;
 }
 
