@@ -87,7 +87,7 @@ static void *pctrl_thread(void *arg) {
         read(ctty, &c, 1);
         switch(tolower(c)) {
             case ' ':
-                pstatus->paused = !pstatus->paused;
+                pstatus->paused ^= -1;
                 break;
             case '<':
                 pstatus->brk = PCTRL_PREV;
@@ -98,6 +98,9 @@ static void *pctrl_thread(void *arg) {
             case 'c':
                 pstatus->factor *= 1.33;
                 break;
+            case 'e':
+                pstatus->echo ^= -1;
+                break;
             case '6':
                 if(pstatus->skval < 0) { pstatus->skval = 0; }
                 pstatus->paused = 0;
@@ -123,6 +126,7 @@ static void *pctrl_thread(void *arg) {
         }
         pthread_mutex_unlock(&ps_lock);
     }
+
     return NULL;
 }
 
