@@ -37,10 +37,6 @@ user/rdsh.c - Shared functions for RPLD/INFOD/RPLCTL
 #include "global.h"
 #include "rdsh.h"
 
-// Functions
-inline static int K_MINORBITS(void);
-static void setup_kminorbits(void);
-
 // External Variables
 pthread_mutex_t Ttys_lock = PTHREAD_MUTEX_INITIALIZER;
 struct HXbtree *Ttys = NULL;
@@ -50,21 +46,18 @@ struct GOptmap_t GOpt = {
     .syslog     = 0,
     .verbose    = 0,
     .infod_port = "/var/run/.rplinfod_socket",
+    .ofmt       = "%u/%d.%t.%l", // -O
 };
 
-// Variables
-static unsigned char kver = 0;
-static pthread_mutex_t kver_lock = PTHREAD_MUTEX_INITIALIZER;
-
 //-----------------------------------------------------------------------------
 static const char *m_idx = "pqrstuvwxyzabcdef", *s_idx = "0123456789abcdef";
 
 char *G_devname_nm(uint32_t dev, char *buf, size_t count) {
     /* This function returns a "simple name" which can be integrated into
     rpld's log file name scheme. */
-    register unsigned long major = K26_MAJOR(dev), minor = K26_MINOR(dev);
+    register unsigned long minor = K26_MINOR(dev);
 
-    switch(major) {
+    switch(K26_MAJOR(dev)) {
         case 2: // BSD pty masters
             snprintf(buf, count, "pty%c%c",
              m_idx[minor >> 4], s_idx[minor % 16]);
@@ -81,21 +74,12 @@ char *G_devname_nm(uint32_t dev, char *buf, size_t count) {
         case 5:
             if(minor == 1) { snprintf(buf, count, "console"); }
             else { goto unknown; }
-        case 128 ... 135:
+        case 128:
             // Unix98 pty masters (got no real /dev entry, though, see below)
-            snprintf(buf, count, "ptm-%lu",
-             ((major - 128) << K_MINORBITS()) | minor);
+            snprintf(buf, count, "ptm-%lu", minor);
             break;
-        case 136 ... 143: // Unix98 pty slaves
-            /* Surprise, surprise. In 2.4, Unix98 ptys use major 136..143 while
-            in 2.6 only 136 thanks to the bigger minor numbers. Anyway, can't
-            be bad to have support for 136..143 for 2.6 ;-) */
-            snprintf(buf, count, "pts-%lu",
-             ((major - 136) << K_MINORBITS()) | minor);
-            /* Thus: 2.4(137,2) == 2.6(136, 258)
-               And: 2.6(137,2) == 2.6(136, 1048578)... kinda
-            Same weirdance goes for pty masters. Lucky BSD, ever since only
-            got 256 devices fitting in one major, and still at it :-D */
+        case 136: // Unix98 pty slaves
+            snprintf(buf, count, "pts-%lu", minor);
             break;
         default:
             goto unknown;
@@ -112,7 +96,7 @@ unknown:
 char *G_devname_fs(uint32_t dev, char *buf, size_t count) {
     /* This function returns a device filename generated from major/minor
     number, which is then used for stat(). */
-    register unsigned long major = K26_MAJOR(dev), minor = K26_MINOR(dev);
+    register unsigned long minor = K26_MINOR(dev);
 
     switch(K26_MAJOR(dev)) {
         case 2: // BSD pty masters
@@ -135,12 +119,10 @@ char *G_devname_fs(uint32_t dev, char *buf, size_t count) {
             /* Unix98 pty masters. They usually have no device nodes in /dev,
             and should instead be accessed over the multiplexer /dev/ptmx.
             However IIRC, devfs popped them up in /dev/ptm. */
-            snprintf(buf, count, "/dev/ptm/%lu",
-             ((major - 128) << K_MINORBITS()) | minor);
+            snprintf(buf, count, "/dev/ptm/%lu", minor);
             break;
         case 136 ... 143: // Unix98 pty slaves
-            snprintf(buf, count, "/dev/pts/%lu",
-             ((major - 136) << K_MINORBITS()) | minor);
+            snprintf(buf, count, "/dev/pts/%lu", minor);
             break;
         default:
             return NULL;
@@ -164,7 +146,7 @@ struct tty *get_tty(uint32_t dev) {
 
     if((ts = HXbtree_add(Ttys, (void *)dev, tty)) == NULL) {
         free(tty);
-        notify(LOG_ERR, NF_NEWLINE, "Memory allocation failure");
+        notify(LOG_ERR, "%s: Memory allocation failure\n", __FUNCTION__);
         return NULL;
     }
 
@@ -173,9 +155,7 @@ struct tty *get_tty(uint32_t dev) {
 
 void log_close(struct tty *tty) {
     /* Close the logfile and release the tty struct if it does not have special
-    options are set. */
-    pthread_mutex_lock(&Ttys_lock);
-
+    options have set. */
     close(tty->fd);
     tty->fd = -1;
     if(tty->file != NULL) {
@@ -191,28 +171,26 @@ void log_close(struct tty *tty) {
         HXbtree_del(Ttys, (void *)tty->dev);
         free(tty);
     }
-    pthread_mutex_unlock(&Ttys_lock);
     return;
 }
 
-int notify(int lv, unsigned int opts, const char *fmt, ...) {
+void notify(int lv, const char *fmt, ...) {
     if(GOpt.verbose) {
-        int rv;
         va_list argp;
         va_start(argp, fmt);
-        if(opts & NF_NEWLINE) { fprintf(stderr, "\n"); }
-        rv = vfprintf(stderr, fmt, argp);
+        fprintf(stderr, "\n");
+        vfprintf(stderr, fmt, argp);
         va_end(argp);
-        return rv;
+        return; // do not print to syslog if we do to stdout
     }
     if(GOpt.syslog) {
         va_list argp;
         va_start(argp, fmt);
         vsyslog(lv, fmt, argp);
         va_end(argp);
-        return 1;
+        return;
     }
-    return 0;
+    return;
 }
 
 ssize_t send_wait(int fd, const void *buf, size_t count, int flags) {
@@ -228,26 +206,4 @@ ssize_t send_wait(int fd, const void *buf, size_t count, int flags) {
     return count;
 }
 
-//-----------------------------------------------------------------------------
-inline static int K_MINORBITS(void) {
-    pthread_mutex_lock(&kver_lock);
-    if(kver == 0) { setup_kminorbits(); }
-    pthread_mutex_unlock(&kver_lock);
-    return (kver != 24) ? K26_MINORBITS : K24_MINORBITS;
-}
-
-static void setup_kminorbits(void) {
-    char buf[64];
-    FILE *fp;
-
-    memset(&buf, '\0', 64);
-    if((fp = fopen("/proc/version", "r")) == NULL) { return; }
-
-    fscanf(fp, "%*s %*s %60s", buf);
-    if(strncmp(buf, "2.6", 3) == 0) { kver = 26; }
-    else if(strncmp(buf, "2.4", 3) == 0) { kver = 24; }
-    fclose(fp);
-    return;
-}
-
 //==[ End of file ]============================================================
