@@ -70,6 +70,8 @@ struct {
 
 // Functions
 static int replay_file(int, const char *);
+static void e_proc(int, struct disk_packet *, struct pctrl_info *, char *,
+  struct timeval *, unsigned long *);
 
 static unsigned long calc_ovcorr(unsigned long, int);
 static int find_next_packet(int);
@@ -101,8 +103,8 @@ int main(int argc, const char **argv) {
 
     if(argc == 0) {
         if(!isatty(STDIN_FILENO)) {
-            fprintf(stderr, "No arguments given and STDIN is not a tty --"
-             " replaying from STDIN\n");
+            fprintf(stderr, "No arguments given but STDIN seems to be a "
+              "pipe/file -- replaying from STDIN\n");
             argv = auto_stdin;
         } else {
             fprintf(stderr, "No filename(s) given.\n");
@@ -110,15 +112,16 @@ int main(int argc, const char **argv) {
         }
     }
 
+    setvbuf(stdout, NULL, _IONBF, 0);
     Opt.ovcorr = calc_ovcorr(0, 100);
-
     if(!Opt.no_pctrl) { pctrl_init(); }
 
-    for(i = 0; i < argc; ) {
+    for(i = 0; i < argc; /* see switch() */) {
         int fd, rv;
+        if(strcmp(argv[i], "-") == 0) { argv[i] = "/dev/stdin"; }
         if((fd = open(argv[i], O_RDONLY)) < 0) {
-            fprintf(stderr, "Could not open %s: %s\n", argv[i],
-             strerror(errno));
+            fprintf(stderr, "Could not open %s: %s\n",
+              argv[i], strerror(errno));
             ++i;
             continue;
         }
@@ -152,12 +155,12 @@ static int replay_file(int fd, const char *name) {
     struct timeval stamp;
     ssize_t eax;
     long skew = 0;
-    int tick = 0;
+    char tick = 0;
 
     if(Opt.follow == FM_LIVE && !seek_to_end(fd)) { return 0; }
 
-
     memset(&ps, 0, sizeof(struct pctrl_info));
+    ps.echo   = 0;
     ps.factor = Opt.factor;
     ps.sktype = Opt.sktype;
     ps.skval  = Opt.skval;
@@ -165,8 +168,6 @@ static int replay_file(int fd, const char *name) {
 
     while((eax = read_waitfm(fd, &packet, sizeof(struct disk_packet))) ==
      sizeof(struct disk_packet)) {
-        struct timeval delta;
-
         if(packet.magic != MAGIC_SIG) {
             fprintf(stderr, "\n" "<Packet inconsistency! "
              "Trying to find next valid packet.>\n");
@@ -183,67 +184,7 @@ static int replay_file(int fd, const char *name) {
 
         switch(packet.event) {
             case EVT_WRITE:
-                if(ps.sktype == PCTRL_SKPACK && ps.skval-- > 0) {
-                    read_nullfm(fd, packet.size);
-                    break;
-                }
-                if(!tick) {
-                    // No delay after the first packet has been read...
-                    ++tick;
-                    if(ps.skval > 0) {
-                        // Urgh, these two lines are replicated below so much
-                        memcpy(&stamp, &packet.tv, sizeof(struct timeval));
-                        read_nullfm(fd, packet.size);
-                        break;
-                    }
-                } else if(Opt.follow != FM_LIVE && Opt.follow != FM_NODELAY) {
-                    /* ... only when we know the next packet and can calculate
-                    the time difference. We do not execute this code in FM_LIVE
-                    mode, because we are already waiting enough with
-                    read_wait(). In fact, allowing this code block for FM_LIVE
-                    creates a skew because of the usleep_ovcorr()! */
-                    unsigned long long x;
-                    tv_delta(&stamp, &packet.tv, &delta);
-                    if(ps.sktype == PCTRL_SKTIME) {
-                        if((ps.skval -= tv2usec(&delta) / 1000) > 0) {
-                            memcpy(&stamp, &packet.tv, sizeof(struct timeval));
-                            //read_nullfm(fd, packet.size);
-                            read_through(fd, STDOUT_FILENO, packet.size);
-                            break;
-                        }
-                    }
-                    if(ps.factor != 1.0) {
-                        // -S switch for the fast
-                        usec2tv(tv2usec(&delta) / ps.factor, &delta);
-                    }
-                    if(Opt.maxdelay > 0 &&
-                     (x = tv2usec(&delta)) > Opt.maxdelay) {
-                        /* -m switch for those who do not want to wait their
-                        lifetime. */
-                        usec2tv(Opt.maxdelay, &delta);
-                    }
-                    usleep_ovcorr(&delta, &skew);
-                }
-
-                while(ps.paused) { sched_yield(); }
-
-                memcpy(&stamp, &packet.tv, sizeof(struct timeval));
-                read_through(fd, STDOUT_FILENO, packet.size);
-
-                if(Opt.showtime) {
-                    struct tm tm;
-                    char buf[80];
-                    time_t now = packet.tv.tv_sec;
-                    localtime_r(&now, &tm);
-                    if(Opt.showtime == 1) {
-                        strftime(buf, sizeof(buf), "\e7\e[1;300H\e[9D"
-                         "\e[0;7m[\e[0;1;37;41m%H:%M:%S\e[0;7m]\e8", &tm);
-                    } else {
-                        strftime(buf, sizeof(buf), "\e7\e[1;300H\e[12D"
-                         "\e[0;7m[\e[0;1;37;41m%d/%m %H:%M\e[0;7m]\e8", &tm);
-                    }
-                    write(STDOUT_FILENO, buf, strlen(buf));
-                }
+                e_proc(fd, &packet, &ps, &tick, &stamp, &skew);
                 break;
             case EVT_IDENT:
                 fprintf(stderr, "\n" "Created using ");
@@ -252,7 +193,17 @@ static int replay_file(int fd, const char *name) {
                 break;
             case EVT_DEINIT:
                 fprintf(stderr, "\n" "<tty device has been closed>\n");
-                // fallthrough
+                // fallthrough kinda
+                read_nullfm(fd, packet.size);
+                break;
+            case EVT_READ:
+                if(ps.echo) {
+                    printf("\e[31m");
+                    e_proc(fd, &packet, &ps, &tick, &stamp, &skew);
+                    printf("\e[0m");
+                    break;
+                }
+                // true fallthrough
             default:
                 read_nullfm(fd, packet.size);
                 break;
@@ -271,6 +222,70 @@ static int replay_file(int fd, const char *name) {
     return ps.brk;
 }
 
+static void e_proc(int fd, struct disk_packet *p, struct pctrl_info *i,
+  char *tick, struct timeval *stamp, unsigned long *skew)
+{
+    if(i->sktype == PCTRL_SKPACK && i->skval-- > 0) {
+        // Just skip, do not update tick/stamp/skew/delta
+        read_through(fd, STDOUT_FILENO, p->size);
+        return;
+    }
+
+    if(!*tick) {
+        // No delay after the first packet has been read...
+        ++*tick;
+        if(i->skval > 0) {
+            memcpy(stamp, &p->tv, sizeof(struct timeval));
+            read_through(fd, STDOUT_FILENO, p->size);
+            return;
+        }
+    } else if(Opt.follow != FM_LIVE && Opt.follow != FM_NODELAY) {
+        /* ... only when we know the next packet (which is already read by now)
+        the tiem difference can be calculated. This code is not executed in
+        FM_LIVE mode, because we have been already waiting enough with
+        read_wait(). */
+        struct timeval delta;
+        tv_delta(stamp, &p->tv, &delta);
+
+        if(i->sktype == PCTRL_SKTIME &&
+         (i->skval -= tv2usec(&delta) / 1000) > 0) {
+            memcpy(stamp, &p->tv, sizeof(struct timeval));
+            read_through(fd, STDOUT_FILENO, p->size);
+            return;
+        }
+        if(i->factor != 1.0) {
+            // -S switch for the fast
+            usec2tv((double)tv2usec(&delta) / i->factor, &delta);
+        }
+        if(Opt.maxdelay > 0 && tv2usec(&delta) > Opt.maxdelay) {
+            // -m switch for those who do not want to wait their lifetime.
+            usec2tv(Opt.maxdelay, &delta);
+        }
+        usleep_ovcorr(&delta, skew);
+    }
+
+    while(i->paused) { sched_yield(); }
+
+    memcpy(stamp, &p->tv, sizeof(struct timeval));
+    read_through(fd, STDOUT_FILENO, p->size);
+
+    if(Opt.showtime) {
+        struct tm tm;
+        char buf[80];
+        time_t now = p->tv.tv_sec;
+        localtime_r(&now, &tm);
+        if(Opt.showtime == 1) {
+            strftime(buf, sizeof(buf), "\e7\e[1;300H\e[9D"
+             "\e[0;7m[\e[0;1;37;41m" "%H:%M:%S" "\e[0;7m]\e8", &tm);
+        } else {
+            strftime(buf, sizeof(buf), "\e7\e[1;300H\e[12D"
+             "\e[0;7m[\e[0;1;37;41m" "%d.%m %H:%M" "\e[0;7m]\e8", &tm);
+        }
+        write(STDOUT_FILENO, buf, strlen(buf));
+    }
+    return;
+}
+
 //-----------------------------------------------------------------------------
 static unsigned long calc_ovcorr(unsigned long ad, int rd) {
     struct timespec s = {.tv_sec = 0, .tv_nsec = ad};
