@@ -44,7 +44,7 @@ user/rplctl.c - INFOD client
 static uint32_t getdev(const char *);
 static void send_int(int, unsigned char, uint32_t);
 static void send_void(int, unsigned char);
-static void send_string(int, unsigned char, const char *);
+//static void send_string(int, unsigned char, const char *);
 static int unix_client(const char *);
 
 static int get_options(int *, const char ***);
@@ -163,6 +163,7 @@ static void send_void(int fd, unsigned char req) {
     return;
 }
 
+/*
 static void send_string(int fd, unsigned char req, const char *str) {
     size_t size = strlen(str);
     uint32_t szpack = size;
@@ -173,6 +174,7 @@ static void send_string(int fd, unsigned char req, const char *str) {
     read_reply(fd);
     return;
 }
+*/
 
 static int unix_client(const char *port) {
     struct sockaddr_un sk;
@@ -198,13 +200,13 @@ static int get_options(int *argc, const char ***argv) {
         {NULL, 'A', POPT_ARG_STRING, &tmp, 'A',
          "Capture data from tty", "tty"},
         {NULL, 'D', POPT_ARG_STRING, &tmp, 'D',
-         "Ignore data from tty until next tty session", "tty"},
+         "Ignore data from tty", "tty"},
         {NULL, 'F', POPT_ARG_NONE, NULL, 'F',
          "Flush tty table", NULL},
-        {NULL, 'J', POPT_ARG_STRING, &tmp, 'J',
-         "Ignore data from tty", "tty"},
         {NULL, 'L', POPT_ARG_NONE, NULL, 'L',
          "Explicitly request info about all ttys", NULL},
+        {NULL, 'S', POPT_ARG_STRING, &tmp, 'S',
+         "Ignore data from tty until next tty session", "tty"},
         {NULL, 'X', POPT_ARG_STRING, &tmp, 'X',
          "Call log_close() for the given tty", "tty"},
         {NULL, 'Z', POPT_ARG_NONE, NULL, 'Z',
@@ -225,16 +227,16 @@ static int get_options(int *argc, const char ***argv) {
     while((c = poptGetNextOpt(ctx)) >= 0) {
         static const int mapping[] = {
             ['A'] = IFP_ACTIVATE,
-            ['D'] = IFP_DEACTIVSES,
+            ['D'] = IFP_DEACTIVATE,
             ['F'] = IFP_FLUSH,
-            ['J'] = IFP_DEACTIVATE,
+            ['S'] = IFP_DEACTIVSES,
             ['X'] = IFP_REMOVE,
             ['Z'] = IFP_ZERO,
         };
         switch(c) {
             case 'A':
             case 'D':
-            case 'J':
+            case 'S':
             case 'X': {
                 uint32_t dev;
                 ++got;
