@@ -52,9 +52,8 @@ extern int posix_memalign(void **, size_t, size_t);
 // The prototype order is the same as the function body order
 static void mainloop(int);
 
-static int packet_action1(struct mem_packet *);
-static int packet_action2(struct mem_packet *, struct tty *);
-static int packet_action3(struct mem_packet *, struct tty *, int);
+static int packet_preprox(struct mem_packet *);
+static int packet_process(struct mem_packet *, struct tty *, int);
 
 static void log_cuid(struct tty *);
 static int log_open(struct tty *);
@@ -83,7 +82,7 @@ static int value_cmp(void *, void *);
 static void *Buffer = NULL;
 static struct {
     int _running, dolog;
-    char *ovdev, *fbdev, *ofmt;
+    char *ovdev, *fbdev;
     long bsize;
 
     int infod_start;
@@ -93,7 +92,6 @@ static struct {
     .ovdev       = NULL,          // -D
     .fbdev       = "/dev/rpl:/dev/misc/rpl",
     .dolog       = 1,             // !-Q
-    .ofmt        = "%u@%d.%t.%l", // -O
     .infod_start = 0,             // -I
 };
 
@@ -102,6 +100,8 @@ int main(int argc, const char **argv) {
     pthread_t infod_id;
     int fd;
 
+    umask(umask(0022) | 0477);
+
     /* Yep, the config file is what is needed by all three
     (rpld, infod, rplctl). */
     if(!read_config("/etc/rpld.conf") && errno != ENOENT) {
@@ -131,8 +131,8 @@ int main(int argc, const char **argv) {
         );
     }
 
-    if((Ttys = HXbtree_init(HXBT_ASSOC | HXBT_LKEY | HXBT_XCPY | HXBT_FCMP,
-     value_cmp)) == NULL) {
+    if((Ttys = HXbtree_init(HXBT_ASSOC | HXBT_LKEY | HXBT_XCPY |
+     HXBT_FCMP, value_cmp)) == NULL) {
         perror("Ttys = HXbtree_init()");
         return EXIT_FAILURE;
     }
@@ -146,17 +146,13 @@ int main(int argc, const char **argv) {
     init_sighandler();
 
     // Callback tick for statistics
-    if(GOpt.verbose) { alarm(1); }
-
+    if(GOpt.verbose)    { alarm(1); }
     if(Opt.infod_start) { infod_init(); }
-
-    if(GOpt.syslog) { openlog("rpld", LOG_PID, LOG_DAEMON); }
+    if(GOpt.syslog)     { openlog("rpld", LOG_PID, LOG_DAEMON); }
 
     // Not seteuid(), because it shall not be posibble to regain privs.
     if(GOpt.user_id >= 0) { setuid(GOpt.user_id); }
-
     if(Opt.infod_start) { pthread_create(&infod_id, NULL, infod_main, NULL); }
-
     mainloop(fd);
 
     if(Opt.infod_start) {
@@ -182,18 +178,13 @@ static void mainloop(int fd) {
         }
 
         PP_swab(&packet.dev, sizeof(packet.dev));
+        //PP_swab(&packet.dev2, sizeof(packet.dev2)); // .dev2 unused ATM
         PP_swab(&packet.size, sizeof(packet.size));
 
-        if(packet.magic == MAGIC_2_6) {
-            // nothing needed
-        } else if(packet.magic == MAGIC_2_4) {
-            packet.magic = MAGIC_2_6;
-            packet.dev   = K26_MKDEV(K24_MAJOR(packet.dev),
-             K24_MINOR(packet.dev));
-        } else {
+        if(packet.magic != MAGIC_2_6) {
             ++Stats.badpack;
-            notify(LOG_WARNING, NF_NEWLINE,
-             "Bogus packet (magic is 0x%X)!\n", packet.magic);
+            notify(LOG_WARNING, "Bogus packet (magic is 0x%02X)!\n",
+             packet.magic);
             continue;
         }
 
@@ -201,109 +192,140 @@ static void mainloop(int fd) {
         the amount of data which has to be copied over the device. */
         gettimeofday(&packet.tv, NULL);
 
-        if(!packet_action1(&packet)) {
+        if(!packet_preprox(&packet)) {
             G_skip(fd, packet.size, 0);
             continue;
         }
 
         pthread_mutex_lock(&Ttys_lock);
-        tty = get_tty(packet.dev);
-        pthread_mutex_unlock(&Ttys_lock);
-
-        if(tty == NULL || !packet_action2(&packet, tty)) {
+        if((tty = get_tty(packet.dev)) == NULL) {
             G_skip(fd, packet.size, 0);
+            pthread_mutex_unlock(&Ttys_lock);
             continue;
         }
 
-        packet_action3(&packet, tty, fd);
+        if(!packet_process(&packet, tty, fd)) {
+            /* packet_process is always a success, but it returns 0 to indicate
+            if it wnats to skip the payload. */
+            G_skip(fd, packet.size, 0);
+        }
+        pthread_mutex_unlock(&Ttys_lock);
     }
 
     return;
 }
 
 //-----------------------------------------------------------------------------
-static int packet_action1(struct mem_packet *packet) {
-    /* pty masters are an exact copy of their slaves,
-    so no need to record them too. */
+static int packet_preprox(struct mem_packet *packet) {
+    static int *tab[] = {
+        [EVT_INIT]   = &Stats.init,
+        [EVT_OPEN]   = &Stats.open,
+        [EVT_READ]   = &Stats.read,
+        [EVT_WRITE]  = &Stats.write,
+        [EVT_CLOSE]  = &Stats.close,
+        [EVT_DEINIT] = &Stats.deinit,
+        [EVT_IOCTL]  = &Stats.ioctl,
+        [EVT_max]    = NULL,
+    };
+
+    if(packet->event < EVT_max && tab[packet->event] != NULL) {
+        ++*tab[packet->event];
+    }
+
+    // General packet classification (first stage drop)
+    switch(packet->event) {
+        // Not used in rpld ATM
+        case EVT_INIT:
+        case EVT_CLOSE:
+        case EVT_IOCTL:
+            return 0;
+
+        // These will be processed
+        case EVT_OPEN:
+        case EVT_DEINIT:
+            break;
+
+        // The following roll their own + will be processed...
+        case EVT_READ:
+            Stats.in += packet->size;
+            break;
+        case EVT_WRITE:
+            Stats.out += packet->size;
+            break;
+        default:
+            notify(LOG_WARNING, "Unknown packet type 0x%02X\n", packet->event);
+            return 0;
+    }
+
     {
         register long maj = K26_MAJOR(packet->dev);
+        if(maj == 128 /* Unix98 ptm (-) */) {
+            if(packet->event != EVT_DEINIT) {
+                /* All master side events (except EVT_DEINIT) are dropped,
+                since they are pty masters are an exact copy of their slaves,
+                so no need to record them too. */
+                return 0;
+            }
 
-        if(maj >= 128 && maj < 136 /* Unix98 ptm (-) */) {
-            if(packet->event != EVT_DEINIT) { return 0; }
-            /* When closing a pty, the EVT_DEINIT is generated on the master
-            side. To keep it easy, it's device signature is simply changed to
-            the slave. */
-            packet->dev = K26_MKDEV(K26_MAJOR(packet->dev) + 8,
-             K26_MINOR(packet->dev));
+            /* What's up with the ptm/pts stuffage:
+            Opening an xterm / screen window (with some debug printf in rpld):
+              EVT_INIT   dev=128:5 dev2=136:5 size=0
+              EVT_OPEN   dev=136:5 dev2=136:5 size=0
+
+            Closing it:
+              EVT_CLOSE  dev=136:5 dev2=128:5 size=0
+              EVT_CLOSE  dev=128:5 dev2=136:5 size=0
+              EVT_DEINIT dev=128:5 dev2=136:5 size=0
+
+            But killing the screen using ^A^K:
+              EVT_CLOSE  dev=128:5 dev2=136:5 size=0
+              EVT_CLOSE  dev=136:5 dev2=128:5 size=0
+              EVT_DEINIT dev=136:5 dev2=128:5 size=0
+
+            Not all CLOSEs and DEINITs are synchronous. Swapping the devs so
+            that dev always is 136: seems to be a good idea. And it works. */
+
+            uint32_t tmp = packet->dev2;
+            packet->dev2 = packet->dev;
+            packet->dev  = tmp;
         } else if(maj == 2 /* BSD ptm (/dev/ptypXY) */) {
+            /* Honestly, I am not sure whether BSD ptms also behave like Unix98
+            ptms above, but I assume it. So change major 2 to major 3. */
             if(packet->event != EVT_DEINIT) { return 0; }
             packet->dev = K26_MKDEV(K26_MAJOR(3), K26_MINOR(packet->dev));
         }
     }
 
-    /* The excluded ptm packets above are not counted at all, whereas
-    "ignored" packets are. See below for ignore rules. */
-    {
-        static int *tab[] = {
-            [EVT_INIT]   = &Stats.init,
-            [EVT_OPEN]   = &Stats.open,
-            [EVT_READ]   = &Stats.read,
-            [EVT_WRITE]  = &Stats.write,
-            [EVT_IOCTL]  = &Stats.ioctl,
-            [EVT_CLOSE]  = &Stats.close,
-            [EVT_DEINIT] = &Stats.deinit,
-        };
-        register int e = packet->event;
-
-        if(e >= sizeof(tab) / sizeof(int *) || tab[e] == NULL) {
-            notify(LOG_WARNING, NF_NEWLINE, "Unknown packet type 0x%X\n",
-             packet->event);
-            return 0;
-        }
-
-        if(e == EVT_READ) { Stats.in += packet->size; }
-        else if(e == EVT_WRITE) { Stats.out += packet->size; }
-
-        ++*tab[e];
-        if(e == EVT_IOCTL) { return 0; } // no use for ioctls ATM
-    }
-
     return 1;
 }
 
-static int packet_action2(struct mem_packet *packet, struct tty *tty) {
-    /* The name is wrong when you look at the caller, since postaction()
-    actually comes before packet_action(). Anyway, it's post the get_tty(). */
-    if(tty->status != IFP_ACTIVATE) {
-        return 0;
-    }
-    switch(packet->event) {
-        case EVT_READ:
-            tty->in += packet->size;
-            break;
-        case EVT_WRITE:
-            tty->out += packet->size;
-            break;
-    }
-    return 1;
-}
+static int packet_process(struct mem_packet *packet, struct tty *tty, int fd) {
+    if(tty->status != IFP_ACTIVATE) { return 0; }
 
-static int packet_action3(struct mem_packet *packet, struct tty *tty, int fd) {
     switch(packet->event) {
         case EVT_OPEN:
-            /* EVT_OPEN is not handled since a logfile will automatically be
-            opened in log_write() if necessary. */
+            /* EVT_OPEN will not handle logfile opening since log_write() will
+            do that when a EVT_{READ,WRITE} pops up. */
             log_cuid(tty);
             break;
         case EVT_READ:
-        case EVT_WRITE:
+            tty->in += packet->size;
             log_write(packet, tty, fd);
-            break;
+            return 1;
+        case EVT_WRITE:
+            tty->out += packet->size;
+            log_write(packet, tty, fd);
+            return 1;
         case EVT_DEINIT:
             log_close(tty);
             break;
+        default:
+            notify(LOG_ERR, "Should never get here! (%s:%d) Forgot to code"
+             " something? (event=%d)\n", __FILE__, __LINE__, packet->event);
+            break;
     }
-    return 1;
+
+    return 0;
 }
 
 //-----------------------------------------------------------------------------
@@ -320,6 +342,17 @@ static void log_cuid(struct tty *tty) {
     return;
 }
 
+static int check_parent_directory(const char *s) {
+    char *path = alloca(strlen(s) + 1), *p;
+    strcpy(path, s);
+    if((p = strrchr(path, '/')) == NULL) {
+        // Current dirctory, no more dir checks needed
+        return 1;
+    }
+    *p = '\0';
+    return HX_mkdir(path); // like `mkdir -p`
+}
+
 static int log_open(struct tty *tty) {
     char buf[MAXFNLEN], devnode[32], fmday[16], fmtime[16], user[32];
     struct HX_repmap catalog[] = {{'d', NULL, fmday}, {'l', NULL, devnode},
@@ -348,17 +381,21 @@ static int log_open(struct tty *tty) {
 
     // Simple tty name
     G_devname_nm(tty->dev, devnode, sizeof(devnode));
-    HX_strrep(buf, MAXFNLEN, Opt.ofmt, catalog);
+    HX_strrep(buf, MAXFNLEN, GOpt.ofmt, catalog);
 
     // Open log
-    HX_strclone(&tty->file, basename(buf));
-    if(!Opt.dolog) { snprintf(buf, MAXFNLEN, "/dev/null"); }
-    if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, 0600)) < 0) {
-        notify(LOG_ERR, NF_NEWLINE, "Could not open %s: %s\n",
-         buf, strerror(errno));
+    HX_strclone(&tty->file, buf);
+    if(!Opt.dolog) {
+        snprintf(buf, MAXFNLEN, "/dev/null");
+    } else if(check_parent_directory(buf) <= 0) {
+        notify(LOG_ERR, "Directory permission denied: It won't be possible to"
+         " write to the file %s, expect warnings.\n", buf);
+    }
+    if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, 0200)) < 0) {
+        notify(LOG_ERR, "Could not open %s: %s\n", buf, strerror(errno));
     }
 
-    // ident header (optional)
+    // Ident header (optional)
     if(tty->fd >= 0) {
         struct disk_packet p = {
             .event = EVT_IDENT,
@@ -381,22 +418,18 @@ static int log_open(struct tty *tty) {
 static void log_write(struct mem_packet *packet, struct tty *tty, int fd) {
     /* Slap a timestamp around the packet and write it out to disk. Open the
     log if necessary. */
-    size_t s = packet->size;
-    int read_left;
+    char *buffer = alloca(packet->size);
+    int have;
 
     if(tty->fd == -1) { log_open(tty); }
+    if((have = read(fd, buffer, packet->size)) <= 0) { return; }
+    if(have != packet->size) { packet->size = have; }
+
     PP_swab(&packet->size, sizeof(packet->size));
     PP_swab(&packet->tv.tv_sec, sizeof(packet->tv.tv_sec));
     PP_swab(&packet->tv.tv_usec, sizeof(packet->tv.tv_usec));
     write(tty->fd, &packet->size, sizeof(struct disk_packet));
-    read_left = s;
-
-    while(read_left > 0) { // XXX replace with read_through
-        int have_read = read(fd, Buffer, min_uint(read_left, Opt.bsize));
-        write(tty->fd, Buffer, have_read);
-        read_left -= have_read;
-    }
-
+    write(tty->fd, buffer, have);
     return;
 }
 
@@ -553,7 +586,7 @@ static void sighandler_int(int s) {
 
 static void sighandler_alrm(int s) {
     fprintf(stderr, "\r\e[2K" "IOCD: %zu/%zu/%zu/%zu  RW: %zu/%zu (%llu/%llu)"
-      " C: %zu  B: %zu",
+      " I: %zu  B: %zu",
       Stats.init, Stats.open, Stats.close, Stats.deinit, Stats.read, Stats.write, Stats.in,
       Stats.out, Stats.ioctl, Stats.badpack
     );
@@ -573,9 +606,9 @@ static int get_options(int *argc, const char ***argv) {
     struct poptOption options_table[] = {
         {NULL, 'D', POPT_ARG_STRING, &Opt.ovdev, 0,
          "Path to the RPL device", "file"},
-        {NULL, 'I', POPT_ARG_NONE, &Opt.infod_start, 0,
+        {NULL, 'I', POPT_ARG_VAL, &Opt.infod_start, 1,
          "Start INFOD subcomponent", NULL},
-        {NULL, 'O', POPT_ARG_STRING, &Opt.ofmt, 0,
+        {NULL, 'O', POPT_ARG_STRING, &GOpt.ofmt, 0,
          "Override OFMT variable", "string"},
         {NULL, 'Q', POPT_ARG_NONE, NULL, 'Q',
          "Deactivate logging, only do bytecounts", NULL},
@@ -583,10 +616,12 @@ static int get_options(int *argc, const char ***argv) {
          "User to change to", "user"},
         {NULL, 'c', POPT_ARG_STRING, &tmp, 'c',
          "Read configuration from file (on top of hardcoded conf)", "file"},
+        {NULL, 'i', POPT_ARG_VAL, &Opt.infod_start, 0,
+         "Do not start INFOD subcomponent", NULL},
         {NULL, 's', POPT_ARG_NONE, &GOpt.syslog, 0,
          "Print warnings/errors to syslog", NULL},
         {NULL, 'v', POPT_ARG_NONE, &GOpt.verbose, 0,
-         "Print statistics while rpld is running", NULL},
+         "Print statistics while rpld is running (overrides -s)", NULL},
         POPT_AUTOHELP
         POPT_TABLEEND
     };
@@ -634,7 +669,7 @@ static int read_config(const char *file) {
     static struct rconfig_opt options_table[] = {
         {"OVDEVICE", RCONF_STRING, &Opt.ovdev, NULL},
         {"FBDEVICE", RCONF_STRING, &Opt.fbdev, NULL},
-        {"OFMT",     RCONF_STRING, &Opt.ofmt,  NULL},
+        {"OFMT",     RCONF_STRING, &GOpt.ofmt, NULL},
         {"DO_LOG",   RCONF_IBOOL,  &Opt.dolog, NULL},
         {"USER",     RCONF_CB,     NULL,       pconfig_user},
 
@@ -670,8 +705,8 @@ static int pconfig_user(const char *key, unsigned char type, void *ptr,
 //-----------------------------------------------------------------------------
 inline static const char *basename(const char *s) {
     // Return file component of a full-path filename
-    const char *p = strrchr(s, '/');
-    if(p != NULL) { return p + 1; }
+    const char *p;
+    if((p = strrchr(s, '/')) != NULL) { return p + 1; }
     return s;
 }
 
